#region License Information

/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.ResponseObjects;
using SlaveGroup = HeuristicLab.Hive.Contracts.BusinessObjects.SlaveGroupDto;

namespace HeuristicLab.Hive.Server.Core {
  internal class SlaveManager : ISlaveManager {
    private List<SlaveGroup> slaveGroups;

    public SlaveManager() {
      slaveGroups = new List<SlaveGroup>();
    }

    #region ISlaveManager Members

    /// <summary>
    /// Returns all slaves stored in the database
    /// </summary>
    /// <returns></returns>
    public ResponseList<SlaveDto> GetAllSlaves() {
      ResponseList<SlaveDto> response = new ResponseList<SlaveDto>();
      response.List = new List<SlaveDto>(DaoLocator.SlaveDao.FindAll());
      return response;
    }

    /// <summary>
    /// returns all slave groups stored in the database
    /// </summary>
    /// <returns></returns>
    public ResponseList<SlaveGroup> GetAllSlaveGroups() {
      ResponseList<SlaveGroup> response = new ResponseList<SlaveGroup>();

      List<SlaveGroup> allSlaveGroups = new List<SlaveGroup>(DaoLocator.SlaveGroupDao.FindAllWithSubGroupsAndSlaves());
      SlaveGroup emptySlaveGroup = new SlaveGroup();
      IEnumerable<SlaveDto> groupLessSlaves = DaoLocator.SlaveDao.FindAllSlavesWithoutGroup();
      if (groupLessSlaves != null) {
        foreach (SlaveDto currSlave in groupLessSlaves) {
          emptySlaveGroup.Resources.Add(currSlave);
        }
      }
      emptySlaveGroup.Id = Guid.Empty;
      allSlaveGroups.Add(emptySlaveGroup);

      response.List = allSlaveGroups;

      return response;
    }

    /// <summary>
    /// [chn] check usage 
    /// </summary>
    public ResponseList<UpTimeStatisticsDto> GetAllUpTimeStatistics() {
      return new ResponseList<UpTimeStatisticsDto>();
    }

    /// <summary>
    /// Add a slave group into the database
    /// </summary>
    /// <param name="slaveGroup"></param>
    /// <returns></returns>
    public ResponseObject<SlaveGroup> AddSlaveGroup(SlaveGroup slaveGroup) {
      ResponseObject<SlaveGroup> response = new ResponseObject<SlaveGroup>();

      if (slaveGroup.Id != Guid.Empty) {
        //response.Success = false;
        response.StatusMessage = ResponseStatus.AddSlaveGroup_SlaveIdMustNotBeSet;
      }
      else {
        slaveGroup = DaoLocator.SlaveGroupDao.Insert(slaveGroup);
        //slaveGroupAdapter.Update(slaveGroup);
        response.Obj = slaveGroup;
      }

      return response;
    }

    /// <summary>
    ///  Add a resource to a group
    /// </summary>
    /// <param name="slaveGroupId"></param>
    /// <param name="resource"></param>
    /// <returns></returns>
    public Response AddResourceToGroup(Guid slaveGroupId, ResourceDto resource) {
      Response response = new Response();

      SlaveGroup slaveGroup = DaoLocator.SlaveGroupDao.FindById(slaveGroupId);
      if (slaveGroup == null) {
        //response.Success = false;
        response.StatusMessage = ResponseStatus.AddResourceToGroup_SlaveGroupDoesNotExist;
        return response;
      }

      slaveGroup.Resources.Add(resource);
      DaoLocator.SlaveGroupDao.AddRessourceToSlaveGroup(resource.Id, slaveGroup.Id);

      //If our resource is in fact a slave => set the callendar from the resource as current one.
      
      SlaveDto dbr = DaoLocator.SlaveDao.FindById(resource.Id);
      if(dbr != null) {
        DaoLocator.SlaveDao.SetServerSideCalendar(dbr, slaveGroup.Id);  
      }

      return response;
    }

    /// <summary>
    /// Remove a resource from a group
    /// </summary>
    /// <param name="slaveGroupId"></param>
    /// <param name="resourceId"></param>
    /// <returns></returns>
    public Response DeleteResourceFromGroup(Guid slaveGroupId, Guid resourceId) {
      Response response = new Response();

      SlaveGroup slaveGroup = DaoLocator.SlaveGroupDao.FindById(slaveGroupId);
      if (slaveGroup == null) {
        // response.Success = false;
        response.StatusMessage = ResponseStatus.AddResourceToGroup_SlaveGroupDoesNotExist;
        return response;
      }
     
      DaoLocator.SlaveGroupDao.RemoveRessourceFromSlaveGroup(resourceId, slaveGroup.Id);
      return response;                
    }

    public ResponseObject<SlaveGroupDtoList> GetAllGroupsOfResource(Guid resourceId) {
      ResponseObject<SlaveGroupDtoList> response = new ResponseObject<SlaveGroupDtoList>();

      SlaveDto slave = DaoLocator.SlaveDao.FindById(resourceId);
      if (slave != null) {
        SlaveGroupDtoList groupsOfSlave = new SlaveGroupDtoList(DaoLocator.SlaveGroupDao.MemberOf(slave));
        response.Obj = groupsOfSlave;
      } else {
        response.Obj = new SlaveGroupDtoList();
        //response.Success = false;
        response.StatusMessage = ResponseStatus.GetAllGroupsOfResource_ResourceDoesNotExist;
      }

      return response;
    }

    public Response DeleteSlaveGroup(Guid slaveGroupId) {
      Response response = new Response();

      SlaveGroup slaveGroup = DaoLocator.SlaveGroupDao.FindById(slaveGroupId);
      if (slaveGroup == null) {
        response.StatusMessage = ResponseStatus.DeleteSlaveGroup_SlaveGroupDoesNotExist;
        return response;
      }

      DaoLocator.SlaveGroupDao.Delete(slaveGroup.Id);
      return response;
    }

    public ResponseList<AppointmentDto> GetUptimeCalendarForResource(Guid guid) {
      ResponseList<AppointmentDto> response = new ResponseList<AppointmentDto>();
      response.List = new List<AppointmentDto>(DaoLocator.UptimeCalendarDao.GetUptimeCalendarForResource(guid));
      return response;
    }

    public Response SetUptimeCalendarForResource(Guid guid, IEnumerable<AppointmentDto> appointments, bool isForced) {
      DaoLocator.UptimeCalendarDao.SetUptimeCalendarForResource(guid, appointments);
      DaoLocator.UptimeCalendarDao.NotifySlavesOfNewCalendar(guid, isForced);
      return new Response();
    }
    #endregion
 }
}